%using Babel;
%using Babel.Parser;
%using System.Linq;

%namespace Babel.Lexer


%x COMMENT
%s SQLLIT

	


%{

	public static string GetDescriptionForTokenValue(string token)
	{

		var keyword = GetTokenByText(token);

         if (keyword != null && !string.IsNullOrEmpty(keyword.Description))
			return keyword.Description;

		return string.Empty;
	}


        public class KeyWordDescription
        {
            public Babel.Parser.Tokens Token { get; set; }
            public string Text { get; set; }
            public bool IsCaseSensitive { get; set; }
            public string Description { get; set; }
            public int Glyph {get; set; }
        }

        private static KeyWordDescription[] _KeyWords = null;
        public static KeyWordDescription[] KeyWords
        {
			get {
				if (_KeyWords == null)
				{
					_KeyWords =
						new[] {
						new KeyWordDescription {Token = Tokens.KWBREAK, Text = "break", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWCONTINUE, Text = "continue", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWELSE, Text = "else", IsCaseSensitive = true, Glyph=205 },
						new KeyWordDescription {Token = Tokens.KWIF, Text = "if", IsCaseSensitive = true, Glyph=205 },
						new KeyWordDescription {Token = Tokens.KWFOR, Text = "for", IsCaseSensitive = true, Glyph=205 },
						new KeyWordDescription {Token = Tokens.KWWHILE, Text = "while", IsCaseSensitive = true, Glyph=205 },
						new KeyWordDescription {Token = Tokens.KWRETURN, Text = "return", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWCLASS, Text = "class", IsCaseSensitive = true, Glyph=205 },
						new KeyWordDescription {Token = Tokens.KWPRIVATE, Text = "private", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWPROTECTED, Text = "protected", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWPUBLIC, Text = "public", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWGLOBAL, Text = "global", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWVIRTUAL, Text = "virtual", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWABSTRACT, Text = "abstract", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWWITH, Text = "with", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWWITHOUT, Text = "without", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWSHARING, Text = "sharing", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWWEBSERVICE, Text = "webService", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWENUM, Text = "enum", IsCaseSensitive = true, Glyph=206 },


						new KeyWordDescription {Token = Tokens.KWEXTENDS, Text = "extends", IsCaseSensitive = true, Glyph=206 },


						new KeyWordDescription {Token = Tokens.KWINTEGER, Text = "Integer", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWVOID, Text = "void", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWSTRING, Text = "String", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWDECIMAL, Text = "Decimal", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWDOUBLE, Text = "Double", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWDATE, Text = "Date", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWTIME, Text = "Time", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWDATETIME, Text = "DateTime", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWOBJECT, Text = "object", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWLONG, Text = "Long", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWID, Text = "Id", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWBOOLEAN, Text = "Boolean", IsCaseSensitive = true },

						new KeyWordDescription {Token = Tokens.KWLIST, Text = "List", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWSET, Text = "Set", IsCaseSensitive = true },
						new KeyWordDescription {Token = Tokens.KWMAP, Text = "Map", IsCaseSensitive = true },
						
						new KeyWordDescription {Token = Tokens.KWTESTMETHOD, Text = "testmethod", IsCaseSensitive = false, Glyph=206 },

						new KeyWordDescription {Token = Tokens.KWTRY, Text = "try", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWCATCH, Text = "catch", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWTHROW, Text = "throw", IsCaseSensitive = true, Glyph=206 },

						new KeyWordDescription {Token = Tokens.KWVALUEOF, Text = "valueOf", IsCaseSensitive = true, Glyph=72 },


						new KeyWordDescription {Token = Tokens.KWNEW, Text = "new", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWNULL, Text = "null", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWTRUE, Text = "true", IsCaseSensitive = true, Glyph=206 },
						new KeyWordDescription {Token = Tokens.KWFALSE, Text = "false", IsCaseSensitive = true, Glyph=206 },

						new KeyWordDescription {Token = Tokens.KWSTATIC, Text = "static", IsCaseSensitive = true, Glyph=206 }
					};
				}
				
				return _KeyWords;
			}
        }


		public static KeyWordDescription GetTokenByText(string txt)
		{
             var keyword = KeyWords.Where(c => 
                     (c.IsCaseSensitive && c.Text.Equals(txt, StringComparison.CurrentCulture)) 
                     ||
                     (!c.IsCaseSensitive && c.Text.Equals(txt, StringComparison.CurrentCultureIgnoreCase)) 
                 ).FirstOrDefault();
                 
             return keyword;

		}

		int GetIdToken(string txt)
		{

			var keyword = GetTokenByText(txt);

             if (keyword != null)
             {

                 if (!keyword.IsCaseSensitive)
                     return (int)keyword.Token;

                 if (keyword.Text.Equals(txt, StringComparison.CurrentCulture))
                     return (int)keyword.Token;
             }
             
             return (int)Tokens.IDENTIFIER;


       }
       
       internal void LoadYylval()
       {
           yylval.str = tokTxt;
           yylloc = new LexLocation(tokLin, tokCol, tokLin, tokECol);
       }
       
       public override void yyerror(string s, params object[] a)
       {
          System.Diagnostics.Debug.Print("error (lex): " + s);
          //if (handler != null) handler.AddError(s, tokLin, tokCol, tokLin, tokECol);
       }
%}


White0          [ \t\r\f\v]
White           {White0}|\n

CmntStart    \/\*
CmntEnd      \*\/
ABStar       [^\*\n]*

SqlStart	\[
SqlEnd		\]
SqlText		[^\]\n|\r\n]*


%%

[a-zA-Z_][a-zA-Z0-9_]*    { return GetIdToken(yytext); }
\-?[0-9]+                    { return (int)Tokens.NUMBER; }
\-?[0-9]+\.[0-9]+            { return (int)Tokens.FLOAT; }
\'[^']*\'	              { return (int)Tokens.STRINGLIT; }
\[[0-9]+\]	              { return (int)Tokens.ARRAYINDEX; }
\[[ \t\f\v]*[a-zA-Z0-9_]+[ \t\f\v]*\]   { return (int)Tokens.ARRAYINDEX; }
\[\]					  { return (int)Tokens.ARRAYMODIFIER; }
;                         { return (int)';';    }
,                         { return (int)',';    }
\(                        { return (int)'(';    }
\)                        { return (int)')';    }
\{                        { return (int)'{';    }
\}                        { return (int)'}';    }
=                         { return (int)'=';    }
\^                        { return (int)'^';    }
\+                        { return (int)'+';    }
\-                        { return (int)'-';    }
\*                        { return (int)'*';    }
\/                        { return (int)'/';    }
\!                        { return (int)Tokens.NOT;    }
==                        { return (int)Tokens.EQ;  }
\!=                       { return (int)Tokens.NEQ;   }
\<\>                      { return (int)Tokens.NEQ;   }
\>                        { return (int)Tokens.GT; }
\>=                       { return (int)Tokens.GTE;    }
\<                        { return (int)Tokens.LT;     }
\<=                       { return (int)Tokens.LTE;    }
\&                        { return (int)'&';    }
\|                        { return (int)'|';    }
\.                        { return (int)'.';    }
\'                        { return (int)'\'';    }
\/\/.*\n				  { return (int)Tokens.LEX_COMMENT; } 
\|\|					  { return (int)Tokens.KWOR; }
\&\& 					  { return (int)Tokens.KWAND; }
\\						  { return (int)'\\'; }
:						  { return (int)':'; }
\+\=					  { return (int)Tokens.PLUSEQUALS; }
\-\=					  { return (int)Tokens.MINUSEQUALS; }
\&\=					  { return (int)Tokens.AMPEQUALS; }
\*\=					  { return (int)Tokens.MULTEQUALS; }
\/\=					  { return (int)Tokens.DIVEQUALS; }
\+\+					  { return (int)Tokens.INCREMENT; }
\-\-					  { return (int)Tokens.DECREMENT; }


{CmntStart}{ABStar}\**{CmntEnd} { return (int)Tokens.LEX_COMMENT; } 
{CmntStart}{ABStar}\**          { BEGIN(COMMENT); return (int)Tokens.LEX_COMMENT; }
<COMMENT>\n                     |                                
<COMMENT>{ABStar}\**            { return (int)Tokens.LEX_COMMENT; }                                
<COMMENT>{ABStar}\**{CmntEnd}   { BEGIN(INITIAL); return (int)Tokens.LEX_COMMENT; }

{SqlStart}{SqlText}{SqlEnd} { return (int)Tokens.SQLLIT; } 
{SqlStart}{SqlText}          { BEGIN(SQLLIT); return (int)Tokens.SQLLIT; }
<SQLLIT>\n                     |                                
<SQLLIT>{SqlText}            { return (int)Tokens.SQLLIT; }                                
<SQLLIT>{SqlText}{SqlEnd}   { BEGIN(INITIAL); return (int)Tokens.SQLLIT; }



{White0}+                  { return (int)Tokens.LEX_WHITE; }
\n                         { return (int)Tokens.LEX_WHITE; }
.                          { yyerror("illegal char");
                             return (int)Tokens.LEX_ERROR; }

%{
                      LoadYylval();
%}

%%

/* .... */
