%using Microsoft.VisualStudio.TextManager.Interop
%namespace Babel.Parser
%valuetype LexValue
%partial

/* %expect 5 */


%union {
    public string str;
}


%{
    ErrorHandler handler = null;
    public void SetHandler(ErrorHandler hdlr) { handler = hdlr; }
    internal void CallHdlr(string msg, LexLocation val)
    {
		System.Diagnostics.Debug.Print("error: " + msg);
        //handler.AddError(msg, val.sLin, val.sCol, val.eCol - val.sCol);
    }
    internal TextSpan MkTSpan(LexLocation s) { return TextSpan(s.sLin, s.sCol, s.eLin, s.eCol); }

    internal void Match(LexLocation lh, LexLocation rh) 
    {
        DefineMatch(MkTSpan(lh), MkTSpan(rh)); 
    }
    
    
%}

%token IDENTIFIER NUMBER STRINGLIT SQLLIT FLOAT
%token KWIF KWELSE KWWHILE KWFOR KWCONTINUE KWBREAK KWRETURN
%token KWEXTERN KWSTATIC KWAUTO KWINT KWVOID KWENUM
%token KWABSTRACT KWDECIMAL KWDOUBLE KWDATE KWDATETIME KWGLOBAL KWINTEGER KWID KWLONG KWOBJECT KWSTRING KWTIME KWVIRTUAL KWWITH KWSHARING KWWITHOUT KWPUBLIC KWPRIVATE KWPROTECTED KWCLASS KWBOOLEAN KWWEBSERVICE KWNEW KWTESTMETHOD KWEXTENDS
%token KWTRY KWCATCH KWTHROW
%token KWTRUE KWFALSE 
%token KWOR KWAND
%token KWNULL 
%token KWLIST KWSET KWMAP KWVALUEOF

%token ARRAYINDEX ARRAYMODIFIER


  // %token ',' ';' '(' ')' '{' '}' '=' 
  // %token '+' '-' '*' '/' '!' '&' '|' '^'

%token EQ NEQ GT GTE LT LTE NOT
%token PLUSEQUALS MINUSEQUALS AMPEQUALS MULTEQUALS DIVEQUALS 
%token INCREMENT DECREMENT

%token maxParseToken 
%token LEX_WHITE LEX_COMMENT LEX_ERROR





%left '*' '/'
%left '+' '-'
%%

Program
    : Declarations 
    ;

Declarations
    : ClassDeclaration Declarations { System.Diagnostics.Debug.Print("declaration1"); }
    | ClassDeclaration         { System.Diagnostics.Debug.Print("declaration2"); } 
    | /* empty */  { System.Diagnostics.Debug.Print("declaration3"); }
    ;
    
ClassDeclaration   
    : ClassAccess KWCLASS ClassName ClassBlock   { System.Diagnostics.Debug.Print("classdec1"); }
    | ClassAccess ClassModifier KWCLASS ClassName ClassBlock  { System.Diagnostics.Debug.Print("classdec2"); }                                                                                   
    | ClassModifier KWCLASS ClassName ClassBlock    { System.Diagnostics.Debug.Print("classdec3"); }                                                                
    | ClassAccess KWCLASS ClassName KWEXTENDS ClassName ClassBlock  { System.Diagnostics.Debug.Print("classdec4"); }
    | ClassAccess ClassModifier KWCLASS ClassName KWEXTENDS ClassName ClassBlock   { System.Diagnostics.Debug.Print("classdec5"); }                                                                                  
    | ClassModifier KWCLASS ClassName KWEXTENDS ClassName ClassBlock    { System.Diagnostics.Debug.Print("classdec6"); }                                                                                 
    | KWCLASS ClassName ClassBlock     { System.Diagnostics.Debug.Print("classdec7"); }                                                                                    
    ;

ClassName : IDENTIFIER;

ClassAccess
	: KWGLOBAL
	| KWPUBLIC
	| KWPRIVATE
	;
    
ClassModifier
	: KWABSTRACT
	| KWSTATIC
	;

ClassBlock
    : OpenBlock CloseBlock      { System.Diagnostics.Debug.Print("classblock1"); Match(@1, @2); }
    | OpenBlock ClassBlockContent1 CloseBlock
                                { System.Diagnostics.Debug.Print("classblock2"); Match(@1, @3); }
    | OpenBlock ClassBlockContent1 error 
                                { System.Diagnostics.Debug.Print("classblock3"); CallHdlr("missing '}'", @3); }
    | OpenBlock error CloseBlock
                                { System.Diagnostics.Debug.Print("classblock4"); Match(@1, @3); }
    ;

OpenBlock
    : '{'                       { /*  */ }
    ;

CloseBlock
    : '}'                       { /*  */ }
    ;
    
ClassBlockContent1
	: ClassBlockItem ClassBlockContent1 { System.Diagnostics.Debug.Print("classcontent1"); }
	| ClassBlockItem  { System.Diagnostics.Debug.Print("classcontent2"); }
	| /*  */ { System.Diagnostics.Debug.Print("classcontent3"); }
	;

ClassBlockItem
    : MethodDec
    | CtorDec 
	| TestMethodDec
	| EnumDec   
	/* | PropDec  */
	| ClassDeclaration  { System.Diagnostics.Debug.Print("class1"); }  
    ;

MethodDec
	: MethodAccess MethodModifier MethodType IDENTIFIER MethodDeclarationParenParams MethodBlock   { System.Diagnostics.Debug.Print("meth1"); }
	| MethodAccess MethodType IDENTIFIER MethodDeclarationParenParams MethodBlock   { System.Diagnostics.Debug.Print("meth2"); }
	;
	
CtorDec
    : MethodAccess IDENTIFIER MethodDeclarationParenParams MethodBlock    { System.Diagnostics.Debug.Print("ctor1");  }
    | IDENTIFIER MethodDeclarationParenParams MethodBlock   { System.Diagnostics.Debug.Print("ctor2"); }
	;
	
TestMethodDec
    : MethodAccess KWSTATIC KWTESTMETHOD KWVOID IDENTIFIER MethodDeclarationParenParams MethodBlock   { System.Diagnostics.Debug.Print("testmeth1"); }
    | KWSTATIC KWTESTMETHOD KWVOID IDENTIFIER MethodDeclarationParenParams MethodBlock  { System.Diagnostics.Debug.Print("testmeth2"); }
	;
	
EnumDec
	: KWENUM IDENTIFIER EnumBlock { System.Diagnostics.Debug.Print("enum1"); }
	| MethodAccess KWENUM IDENTIFIER EnumBlock { System.Diagnostics.Debug.Print("enum2"); }
	;
	

PropDec
	: MethodAccess VariableType LocalVariableName ';' 
	;
	
	
MethodType
	: VariableType
	| KWVOID
	;

EnumBlock
    : OpenBlock EnumBlockContent CloseBlock
                                { Match(@1, @3); }
    | OpenBlock EnumBlockContent error 
                                { CallHdlr("missing '}'", @3); }

	;
	
EnumBlockContent
	: IDENTIFIER ',' EnumBlockContent
	| IDENTIFIER
	;

MethodAccess
	: KWGLOBAL
	| KWPUBLIC
	| KWPRIVATE
	| KWPROTECTED
	| KWWEBSERVICE
	;
	
MethodModifier
	: KWABSTRACT   { System.Diagnostics.Debug.Print("methmod1"); }  
	| KWVIRTUAL { System.Diagnostics.Debug.Print("methmod2"); }  
	| KWSTATIC { System.Diagnostics.Debug.Print("methmod3"); }  
	;	

VariableType
	: VariableType ARRAYMODIFIER
	| KWINTEGER
	| KWLONG
	| KWOBJECT
	| KWSTRING
	| KWDECIMAL 
	| KWDOUBLE 
	| KWDATE 
	| KWDATETIME
	| KWID
	| KWTIME
	| KWBOOLEAN
	| GenericTypeName LT GT { Match(@2, @3); }
	| GenericTypeName LT KWINTEGER GT { Match(@2, @4); }
	| GenericTypeName LT KWLONG GT { Match(@2, @4); }
	| GenericTypeName LT KWOBJECT GT { Match(@2, @4); }
	| GenericTypeName LT KWSTRING GT { Match(@2, @4); }
	| GenericTypeName LT KWDECIMAL GT { Match(@2, @4); }
	| GenericTypeName LT KWDOUBLE GT { Match(@2, @4); }
	| GenericTypeName LT KWDATE GT { Match(@2, @4); }
	| GenericTypeName LT KWDATETIME GT { Match(@2, @4); }
	| GenericTypeName LT KWID GT { Match(@2, @4); }
	| GenericTypeName LT KWTIME GT { Match(@2, @4); }
	| GenericTypeName LT KWBOOLEAN GT { Match(@2, @4); }
	| GenericTypeName LT IDENTIFIER GT { Match(@2, @4); }
	| ExternalTypeName
	;

GenericTypeName
	: KWMAP | KWLIST | KWSET
	;


MethodDeclarationParenParams
    :  '(' ')'                   { System.Diagnostics.Debug.Print("methodparenparam1");  Match(@1, @2); }
    |  '(' MethodParams ')'           { System.Diagnostics.Debug.Print("methodparenparam2");  Match(@1, @3); }
    |  '(' MethodParams error         { System.Diagnostics.Debug.Print("methodparenparam3");  CallHdlr("unmatched parentheses", @3); }
    |  '(' error ')'             { System.Diagnostics.Debug.Print("methodparenparam4"); $$ = $3; CallHdlr("error in method params", @2); }
    ;	

MethodParams
    : VariableType IDENTIFIER ',' MethodParams { System.Diagnostics.Debug.Print("methodparams1"); }
    | VariableType IDENTIFIER  { System.Diagnostics.Debug.Print("methodparams2"); }
    ;

MethodBlock
    : OpenBlock CloseBlock      {  System.Diagnostics.Debug.Print("methblock1"); Match(@1, @2); }
    | OpenBlock MethodBlockContent CloseBlock
                                { System.Diagnostics.Debug.Print("methblock2");  Match(@1, @3); }
    | OpenBlock MethodBlockContent error 
                                { System.Diagnostics.Debug.Print("methblock3");  CallHdlr("missing '}'", @3); }
    | OpenBlock error CloseBlock
                                { System.Diagnostics.Debug.Print("methblock4");  Match(@1, @3); }
    ;

BasicBlock
    : OpenBlock CloseBlock      { System.Diagnostics.Debug.Print("basicblock1"); Match(@1, @2); }
    | OpenBlock BasicBlockContent CloseBlock
                                { System.Diagnostics.Debug.Print("basicblock2"); Match(@1, @3); }
    | OpenBlock BasicBlockContent error 
                                { System.Diagnostics.Debug.Print("basicblock3"); CallHdlr("missing '}'", @3); }
    | OpenBlock error CloseBlock
                                { System.Diagnostics.Debug.Print("basicblock4"); Match(@1, @3); }
    ;
   
BasicBlockContent
	: Statements;
	
MethodBlockContent
	: Statements;
	
Statements 
    : Statement Statements
    | Statement
    ;
    
 Statement
	: SemiStatement ';'  { System.Diagnostics.Debug.Print("statement1");  }
	| SemiStatement error ';'       { System.Diagnostics.Debug.Print("statement2"); CallHdlr("expected ';'", @2); } 
	| KWWHILE ParenExprAlways Statement { System.Diagnostics.Debug.Print("statement3");  }
	| KWFOR ForHeader Statement  { System.Diagnostics.Debug.Print("statement4");  }
	| KWIF ParenExprAlways Statement  { System.Diagnostics.Debug.Print("statement5");  }
	| KWIF ParenExprAlways Statement KWELSE Statement  { System.Diagnostics.Debug.Print("statement6");  }
	| TryCatchBlock  { System.Diagnostics.Debug.Print("statement7");  }
	| BasicBlock  { System.Diagnostics.Debug.Print("statement8");  }
	;   	

ForHeader
    : '(' ForBlock ')'          { Match(@1, @3); }
    | '(' ForBlock error        { CallHdlr("unmatched parentheses", @3); }
    | '(' error ')'             { Match(@1, @3); 
                                  CallHdlr("error in for", @2); }
    ;

ForBlock
	: ForEachBlock
	| ForBlockIncrement
	;

ForBlockIncrement
	: ';' ';'
	| SimpleAssignment ';' ';' 
	| SimpleAssignment ';' BooleanExpression ';' 
	| SimpleAssignment ';' BooleanExpression ';' NonDeclarationAssignment
	| ';' BooleanExpression ';' 
	| ';' BooleanExpression ';' NonDeclarationAssignment
	| ';' ';' NonDeclarationAssignment
	;

ForEachBlock
	: VariableNameOrDeclaration ':' SqlExpression
	| VariableNameOrDeclaration ':' ExternalVariableName
	;

ParenExprAlways
    : ParenExpr
    | error ')'                 { CallHdlr("error in expr", @1); }
    | error                     { CallHdlr("error in expr", @1); }
    ;

ParenExpr
    : '(' BooleanExpression ')'              { Match(@1, @3); }
    | '(' BooleanExpression error            { CallHdlr("unmatched parentheses", @3); }
    ;

SemiStatement
	: VariableDeclaration { System.Diagnostics.Debug.Print("semiStatement1"); }
	| SimpleAssignment { System.Diagnostics.Debug.Print("semiStatement2"); }
	| MethodCall { System.Diagnostics.Debug.Print("semiStatement3"); }
	| KWRETURN ValueExpression { System.Diagnostics.Debug.Print("semiStatement4"); }
	| KWBREAK { System.Diagnostics.Debug.Print("semiStatement5"); }
	| KWCONTINUE { System.Diagnostics.Debug.Print("semiStatement6"); }
	| KWTHROW ValueExpression { System.Diagnostics.Debug.Print("semiStatement7"); }
	;


VariableDeclaration
	: VariableType LocalVariableName
	;

VariableNameOrDeclaration
	: LocalVariableName
	| ExternalVariableName
	| VariableDeclaration
	;

NonDeclarationAssignment
	: VariableName '=' SqlExpression
	| VariableName '=' KWNULL
	| VariableName '=' ValueExpression
	| VariableName PLUSEQUALS ValueExpression
	| VariableName MINUSEQUALS ValueExpression
	| VariableName AMPEQUALS ValueExpression
	| VariableName MULTEQUALS ValueExpression
	| VariableName DIVEQUALS ValueExpression
	| VariableName INCREMENT
	| VariableName DECREMENT
	;


SimpleAssignment
	: VariableNameOrDeclaration '=' SqlExpression
	| VariableNameOrDeclaration '=' KWNULL
	| VariableNameOrDeclaration '=' ValueExpression
	| VariableNameOrDeclaration PLUSEQUALS ValueExpression
	| VariableNameOrDeclaration MINUSEQUALS ValueExpression
	| VariableNameOrDeclaration AMPEQUALS ValueExpression
	| VariableNameOrDeclaration MULTEQUALS ValueExpression
	| VariableNameOrDeclaration DIVEQUALS ValueExpression
	;

SqlExpression
	: SqlLiteral
	| SqlLiteral '.' KWID
	| SqlLiteral '.' LocalVariableName
	| SqlLiteral '.' LocalVariableName '.' LocalVariableName
	| SqlLiteral '.' LocalVariableName '.' LocalVariableName '.' LocalVariableName
	| SqlLiteral '.' LocalVariableName '.' LocalVariableName '.' LocalVariableName '.' LocalVariableName
	| SqlLiteral ARRAYINDEX
	;

SqlLiteral
	: SQLLIT SqlLiteral
	| LEX_WHITE SqlLiteral  
	| SQLLIT
	;

ValueExpression
	: MathmaticExpression
	| BooleanExpression
	| MethodCall
	| ExternalVariableName
	| NewInstance
	| SqlExpression
	| KWNULL
	;

MathmaticExpression
	: '(' MathmaticExpression ')' { Match(@1, @3); }
	| AddExpr
	;
	
AddExpr
    : MulExpr AddOrSubtractOp AddExpr
    | MulExpr
    ;

AddOrSubtractOp
    : AddOp | '-'
    ;

AddOp
	: '+'
	;

MulExpr
    : Factor MulOp MulExpr
    | Factor 
    ;

MulOp 
    : '*' | '/'
    ;


Factor   
	: '(' AddExpr ')'  { Match(@1, @3); }
	| NUMBER
	| FLOAT
	| STRINGLIT
	| MethodCall
	| ExternalVariableName
	;

	
BooleanExpression
	: '(' BooleanExpression ')'  { Match(@1, @3); }
	| BoolOrExpr
	;
	
BoolOrExpr
    : BoolAndExpr KWOR BoolOrExpr
    | BoolAndExpr
    ;
BoolAndExpr
    : BoolNotFactor KWAND BoolAndExpr
    | BoolNotFactor 
    ;

BoolNotFactor
	: BFactor
	| NOT BFactor
	;

BFactor   
	: '(' BoolOrExpr ')'  { Match(@1, @3); }
	| BooleanValueExp
	;

BooleanValueExp
	: NonBooleanValueExpression
	| BooleanRelation
	;
	
BooleanRelation
	: NonBooleanValueExpression RelOp NonBooleanValueExpression
	;

NonBooleanValueExpression
	: MathmaticExpression
	| MethodCall
	| ExternalVariableName
	| NewInstance
	| SqlLiteral
	| KWNULL
	| BooleanLiteral
	;

RelOp
    : GT | GTE | LT | LTE | EQ | NEQ 
    ;


TryCatchBlock
	: KWTRY BasicBlock CatchBlocks
	;
	
CatchBlocks
	: CatchBlock
	| CatchBlock CatchBlocks
	;
	
CatchBlock
	: KWCATCH BasicBlock
	| KWCATCH '(' ')' BasicBlock { Match(@2, @3); }
	| KWCATCH '(' VariableType ExternalVariableName ')' BasicBlock
	;



NewInstance
	: KWNEW VariableType MethodCallParenParams
	| KWNEW ExternalTypeName ARRAYMODIFIER '{' ArrayItems '}'
	| KWNEW BasicValueType ARRAYMODIFIER '{' ArrayItems '}'
	;

ArrayItems
	: ArrayItems1
	| /* empty */
	;
	
ArrayItems1
	: ValueExpression ',' ArrayItems1
	| ValueExpression
	;
	

MethodCall
	: MethodCall1
	| MethodCall1 '.' ExternalVariableName
	| MethodCall1 '.' MethodCall
	;
	
MethodCall1
	: ExternalMethodName MethodCallParenParams
	| BasicValueType '.' KWVALUEOF MethodCallParenParams
	| BasicValueType '.' IDENTIFIER MethodCallParenParams
	;

MethodCallParenParams
    :  '(' ')'                   { Match(@1, @2); }
    |  '(' MethodCallParams ')'           { Match(@1, @3); }
    |  '(' MethodCallParams error         { CallHdlr("unmatched parentheses", @3); }
    |  '(' error ')'             { $$ = $3;
                                   CallHdlr("error in params", @2); }
    ;

MethodCallParams
	: MethodCallParams ',' ValueExpression
	| MethodCallParams ',' NonDeclarationAssignment
	| ValueExpression
	| NonDeclarationAssignment
	;


BooleanLiteral
	: KWTRUE
	| KWFALSE
	;
	
VariableName
	: ExternalVariableName
	;
	
ExternalVariableName
	: IDENTIFIER '.' ExternalVariableName
	| IDENTIFIER ARRAYINDEX '.' ExternalVariableName
	| IDENTIFIER '.' KWID
	| IDENTIFIER ARRAYINDEX '.' KWID
	| LocalVariableName
	;


LocalVariableName
	: IDENTIFIER
	| IDENTIFIER ARRAYINDEX
	;

ExternalTypeName
	: IDENTIFIER '.' ExternalTypeName
	| IDENTIFIER
	;

ExternalMethodName
	: IDENTIFIER '.' ExternalMethodName
	| IDENTIFIER ARRAYINDEX '.' ExternalMethodName
	| IDENTIFIER
	;

BasicValueType
	: KWINTEGER
	| KWLONG
	| KWOBJECT
	| KWSTRING
	| KWDECIMAL 
	| KWDOUBLE 
	| KWDATE 
	| KWDATETIME
	| KWID
	| KWTIME
	| KWBOOLEAN
	;
	
	
%%

